/*
 * HashCodeProviderWrapper.cs - Wrap a non-generic hash code provider
 *			to turn it into a generic one.
 *
 * Copyright (c) 2003  Southern Storm Software, Pty Ltd
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 */

namespace Generics
{

using System;

public sealed class HashCodeProviderWrapper<T> : IHashCodeProvider<T>
{

	// Internal state.
	private System.Collections.IHashCodeProvider provider;

	// Constructor.
	public HashCodeProviderWrapper
				(System.Collections.IHashCodeProvider provider)
			{
				if(provider == null)
				{
					throw new ArgumentNullException("provider");
				}
				this.provider = provider;
			}

	// Implement the IHashCodeProvider<T> interface.
	public int GetHashCode(T obj)
			{
				return provider.GetHashCode(obj);
			}

}; // class HashCodeProviderWrapper<T>

}; // namespace Generics
